# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import argparse
import io
import sys

from six import PY3
from six import StringIO
from six import text_type
from six.moves.configparser import ConfigParser
from six.moves.configparser import Error

from language_formatters_pre_commit_hooks.utils import remove_trailing_whitespaces_and_set_new_line_ending


def pretty_format_ini(argv=None):
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--autofix',
        action='store_true',
        dest='autofix',
        help='Automatically fixes encountered not-pretty-formatted files',
    )

    parser.add_argument('filenames', nargs='*', help='Filenames to fix')
    args = parser.parse_args(argv)

    status = 0

    for ini_file in set(args.filenames):
        with open(ini_file) as f:
            string_content = ''.join(f.readlines())

        config_parser = ConfigParser()
        try:
            if PY3:  # pragma: no cover # py3+ only
                config_parser.read_string(string_content)
            else:  # pragma: no cover # py27 only
                config_parser.readfp(StringIO(string_content))

            pretty_content = StringIO()
            config_parser.write(pretty_content)

            pretty_content_str = remove_trailing_whitespaces_and_set_new_line_ending(
                pretty_content.getvalue(),
            )

            if string_content != pretty_content_str:
                print('File {} is not pretty-formatted'.format(ini_file))

                if args.autofix:
                    print('Fixing file {}'.format(ini_file))
                    with io.open(ini_file, 'w', encoding='UTF-8') as f:
                        f.write(text_type(pretty_content_str))

                status = 1
        except Error:
            print('Input File {} is not a valid INI file'.format(ini_file))
            return 1

    return status


if __name__ == '__main__':
    sys.exit(pretty_format_ini())



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import argparse
import io
import sys

from toml import dumps
from toml import loads
from toml import TomlDecodeError

from language_formatters_pre_commit_hooks.utils import remove_trailing_whitespaces_and_set_new_line_ending


def pretty_format_toml(argv=None):
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--autofix',
        action='store_true',
        dest='autofix',
        help='Automatically fixes encountered not-pretty-formatted files',
    )

    parser.add_argument('filenames', nargs='*', help='Filenames to fix')
    args = parser.parse_args(argv)

    status = 0

    for toml_file in set(args.filenames):
        with open(toml_file) as f:
            string_content = ''.join(f.readlines())

        try:
            prettified_content = remove_trailing_whitespaces_and_set_new_line_ending(
                dumps(loads(string_content)),
            )

            if string_content != prettified_content:
                print('File {} is not pretty-formatted'.format(toml_file))

                if args.autofix:
                    print('Fixing file {}'.format(toml_file))
                    with io.open(toml_file, 'w', encoding='UTF-8') as f:
                        f.write(prettified_content)

                status = 1
        except TomlDecodeError:
            print('Input File {} is not a valid TOML file'.format(toml_file))
            return 1

    return status


if __name__ == '__main__':
    sys.exit(pretty_format_toml())



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import argparse
import sys

from language_formatters_pre_commit_hooks.pre_conditions import golang_required
from language_formatters_pre_commit_hooks.utils import run_command


@golang_required
def pretty_format_golang(argv=None):
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--autofix',
        action='store_true',
        dest='autofix',
        help='Automatically fixes encountered not-pretty-formatted files',
    )

    parser.add_argument('filenames', nargs='*', help='Filenames to fix')
    args = parser.parse_args(argv)

    status, output = run_command(
        'gofmt{} -l {}'.format(
            ' -w' if args.autofix else '',
            ' '.join(set(args.filenames)),
        ),
    )

    if status != 0:  # pragma: no cover
        print(output)
        return 1

    status = 0
    if output:
        status = 1
        print(
            '{}: {}'.format(
                'The following files have been fixed by gofmt' if args.autofix else 'The following files are not properly formatted',
                ', '.join(output.splitlines()),
            ),
        )

    return status


if __name__ == '__main__':
    sys.exit(pretty_format_golang())



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import pkg_resources


__version__ = pkg_resources.get_distribution('language_formatters_pre_commit_hooks').version



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import os
import shutil
import subprocess
import sys

import requests
from six.moves.urllib.parse import urlparse


def run_command(command):
    print('[cwd={cwd}] Run command: {command}'.format(command=command, cwd=os.getcwd()), file=sys.stderr)
    return_code, output = None, None
    try:
        return_code, output = 0, subprocess.check_output(
            command,
            stderr=subprocess.STDOUT,
            shell=True,
        ).decode('utf-8')
    except subprocess.CalledProcessError as e:
        return_code, output = e.returncode, e.output.decode('utf-8')
    print('[return_code={return_code}] | {output}'.format(return_code=return_code, output=output), file=sys.stderr)
    return return_code, output


def _base_directory():
    # Extracted from pre-commit code:
    # https://github.com/pre-commit/pre-commit/blob/master/pre_commit/store.py
    return os.environ.get('PRE_COMMIT_HOME') or os.path.join(
        os.environ.get('XDG_CACHE_HOME') or os.path.expanduser('~/.cache'),
        'pre-commit',
    )


def download_url(url, file_name=None):
    final_file = os.path.join(
        _base_directory(),
        file_name or os.path.basename(urlparse(url).path),
    )

    if os.path.exists(final_file):
        return final_file

    r = requests.get(url, stream=True)
    tmp_file = '{}_tmp'.format(final_file)
    with open(tmp_file, mode='wb') as f:
        # Copy on a temporary file in case of issues while downloading the file
        shutil.copyfileobj(r.raw, f)

    os.rename(tmp_file, final_file)
    return final_file


def get_modified_files_in_repo():
    _, command_output = run_command(
        'git diff-index --name-status --binary --exit-code --no-ext-diff $(git write-tree) --',
    )
    return {
        line.split()[-1]
        for line in command_output.splitlines()
        if line
    }


def remove_trailing_whitespaces_and_set_new_line_ending(string):
    return '{content}\n'.format(
        content='\n'.join(
            line.rstrip()
            for line in string.splitlines()
        ).rstrip(),
    )



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import argparse
import io
import re
import sys
from sys import maxsize

from ruamel.yaml import YAML
from ruamel.yaml.error import YAMLError
from six import StringIO
from six import text_type


def _process_single_document(document, yaml):
    """Pretty format one YAML document.

    This is needed in order to prevent `ruamel.yaml` to interfere with documents that have primitive types on the document root.
    For more context check https://github.com/macisamuele/language-formatters-pre-commit-hooks/pull/1

    Args:
        document (str): Original document content.
        yaml: YAML library instance.
    Returns:
        Pretty-formatted content (str).
    """
    content = yaml.load(document)
    if isinstance(content, (list, dict)):
        pretty_output = StringIO()
        yaml.dump(content, pretty_output)
        return pretty_output.getvalue()
    else:
        # do not disturb primitive content (unstructured text)
        return str(document)


def pretty_format_yaml(argv=None):
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--autofix',
        action='store_true',
        dest='autofix',
        help='Automatically fixes encountered not-pretty-formatted files',
    )
    parser.add_argument(
        '--indent',
        type=int,
        default='2',
        help=(
            'The number of indent spaces or a string to be used as delimiter'
            ' for indentation level e.g. 4 or "\t" (Default: 2)'
        ),
    )

    parser.add_argument('filenames', nargs='*', help='Filenames to fix')
    args = parser.parse_args(argv)

    status = 0

    yaml = YAML()
    yaml.indent = args.indent
    # Prevent ruamel.yaml to wrap yaml lines
    yaml.width = maxsize

    separator = '---\n'

    for yaml_file in set(args.filenames):
        with open(yaml_file) as f:
            string_content = ''.join(f.readlines())

        # Split multi-document file into individual documents
        #
        # Not using yaml.load_all() because it reformats primitive (non-YAML) content. It removes
        # newline characters.
        separator_pattern = r'^---\s*\n'
        original_docs = re.split(separator_pattern, string_content, flags=re.MULTILINE)

        # A valid multi-document YAML file might starts with the separator.
        # In this case the first document of original docs will be empty and should not be consdered
        if string_content.startswith('---'):
            original_docs = original_docs[1:]

        pretty_docs = []

        try:
            for doc in original_docs:
                content = _process_single_document(doc, yaml)
                if content is not None:
                    pretty_docs.append(content)

            # Start multi-doc file with separator
            pretty_content = '' if len(pretty_docs) == 1 else separator
            pretty_content += separator.join(pretty_docs)

            if string_content != pretty_content:
                print('File {} is not pretty-formatted'.format(yaml_file))

                if args.autofix:
                    print('Fixing file {}'.format(yaml_file))
                    with io.open(yaml_file, 'w', encoding='UTF-8') as f:
                        f.write(text_type(pretty_content))

                status = 1
        except YAMLError:  # pragma: no cover
            print(
                'Input File {} is not a valid YAML file, consider using check-yaml'.format(
                    yaml_file,
                ),
            )
            return 1

    return status


if __name__ == '__main__':
    sys.exit(pretty_format_yaml())



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import argparse
import sys

from language_formatters_pre_commit_hooks.pre_conditions import java_required
from language_formatters_pre_commit_hooks.utils import download_url
from language_formatters_pre_commit_hooks.utils import get_modified_files_in_repo
from language_formatters_pre_commit_hooks.utils import run_command


KTLINT_VERSION = '0.36.0'


def download_kotlin_formatter_jar(version=KTLINT_VERSION):  # pragma: no cover
    def get_url(_version):
        # Links extracted from https://github.com/shyiko/ktlint/
        return \
            'https://github.com/pinterest/ktlint/releases/download/{version}/ktlint'.format(
                version=_version,
            )

    return download_url(get_url(version), 'ktlint{version}.jar'.format(version=version))


@java_required
def pretty_format_kotlin(argv=None):
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--autofix',
        action='store_true',
        dest='autofix',
        help='Automatically fixes encountered not-pretty-formatted files',
    )

    parser.add_argument('filenames', nargs='*', help='Filenames to fix')
    args = parser.parse_args(argv)

    ktlint_jar = download_kotlin_formatter_jar()

    modified_files_pre_kotlin_formatting = get_modified_files_in_repo()

    status, output = run_command(
        'java -jar {} --verbose {} {}'.format(
            ktlint_jar,
            '--format' if args.autofix else '--',
            ' '.join(set(args.filenames)),
        ),
    )

    if output:
        print(output)
        return 1

    # Check all the file modified by the execution of the previous commands
    modified_files_post_kotlin_formatting = get_modified_files_in_repo()
    if modified_files_pre_kotlin_formatting != modified_files_post_kotlin_formatting:
        print(
            '{}: {}'.format(
                'The following files have been fixed by ktlint' if args.autofix else 'The following files are not properly formatted',  # noqa
                ', '.join(sorted(
                    modified_files_post_kotlin_formatting.difference(modified_files_pre_kotlin_formatting),
                )),
            ),
        )
        return 1

    return 0


if __name__ == '__main__':
    sys.exit(pretty_format_kotlin())



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import argparse
import sys
from os import getenv

from language_formatters_pre_commit_hooks.pre_conditions import rust_required
from language_formatters_pre_commit_hooks.utils import run_command


@rust_required
def pretty_format_rust(argv=None):
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--autofix',
        action='store_true',
        dest='autofix',
        help='Automatically fixes encountered not-pretty-formatted files',
    )

    parser.add_argument('filenames', nargs='*', help='Filenames to fix')
    args = parser.parse_args(argv)

    rust_toolchain_version = getenv('RUST_TOOLCHAIN', 'stable')
    # Check
    status_code, output = run_command(
        'cargo +{} fmt -- --check {}'.format(
            rust_toolchain_version,
            ' '.join(set(args.filenames)),
        ),
    )
    not_well_formatted_files = sorted(
        line.split()[2]
        for line in output.splitlines()
        if line.startswith('Diff in ')
    )
    if not_well_formatted_files:
        print(
            '{}: {}'.format(
                'The following files have been fixed by cargo format' if args.autofix else 'The following files are not properly formatted',
                ', '.join(not_well_formatted_files),
            ),
        )
        if args.autofix:
            run_command(
                'cargo +{} fmt -- {}'.format(
                    rust_toolchain_version,
                    ' '.join(not_well_formatted_files),
                ),
            )

    return 1 if status_code != 0 or not_well_formatted_files else 0


if __name__ == '__main__':
    sys.exit(pretty_format_rust())



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

from os import getenv

from decorator import decorator

from language_formatters_pre_commit_hooks.utils import run_command

_DEFAULT_MESSAGE_TEMPLATE = '{required_tool} is required to run this pre-commit hook. ' \
                            'Make sure that you have it installed and available on your path.\n' \
                            'Download/Install URL: {install_url}'


def _assert_command_succeed(command, assertion_error_message):
    status, _ = run_command(command)
    assert status == 0, assertion_error_message


@decorator
def java_required(f, *args, **kwargs):
    _assert_command_succeed(
        command='java -version',
        assertion_error_message=_DEFAULT_MESSAGE_TEMPLATE.format(
            required_tool='JRE',
            install_url='https://www.java.com/en/download/',
        ),
    )
    return f(*args, **kwargs)


@decorator
def golang_required(f, *args, **kwargs):
    _assert_command_succeed(
        command='go version',
        assertion_error_message=_DEFAULT_MESSAGE_TEMPLATE.format(
            required_tool='golang/gofmt',
            install_url='https://golang.org/doc/install#download',
        ),
    )

    return f(*args, **kwargs)


@decorator
def rust_required(f, *args, **kwargs):
    rust_toolchain_version = getenv('RUST_TOOLCHAIN', 'stable')
    _assert_command_succeed(
        command='cargo +{} fmt  -- --version'.format(rust_toolchain_version),
        assertion_error_message=_DEFAULT_MESSAGE_TEMPLATE.format(
            required_tool='rustfmt',
            install_url='https://github.com/rust-lang-nursery/rustfmt#quick-start',
        ),
    )

    return f(*args, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

from setuptools import setup


setup(
    entry_points={
        'console_scripts': [
            'pretty-format-golang = language_formatters_pre_commit_hooks.pretty_format_golang:pretty_format_golang',
            'pretty-format-java = language_formatters_pre_commit_hooks.pretty_format_java:pretty_format_java',
            'pretty-format-kotlin = language_formatters_pre_commit_hooks.pretty_format_kotlin:pretty_format_kotlin',
            'pretty-format-ini = language_formatters_pre_commit_hooks.pretty_format_ini:pretty_format_ini',
            'pretty-format-rust = language_formatters_pre_commit_hooks.pretty_format_rust:pretty_format_rust',
            'pretty-format-toml = language_formatters_pre_commit_hooks.pretty_format_toml:pretty_format_toml',
            'pretty-format-yaml = language_formatters_pre_commit_hooks.pretty_format_yaml:pretty_format_yaml',
        ],
    },
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import argparse
import sys

from language_formatters_pre_commit_hooks.pre_conditions import java_required
from language_formatters_pre_commit_hooks.utils import download_url
from language_formatters_pre_commit_hooks.utils import run_command


GOOGLE_JAVA_FORMATTER_VERSION = '1.8'


def download_google_java_formatter_jar(version=GOOGLE_JAVA_FORMATTER_VERSION):  # pragma: no cover
    def get_url(_version):
        # Links extracted from https://github.com/google/google-java-format/
        return \
            'https://github.com/google/google-java-format/releases/download/' \
            'google-java-format-{version}/google-java-format-{version}-all-deps.jar'.format(
                version=_version,
            )

    return download_url(get_url(version))


@java_required
def pretty_format_java(argv=None):
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--autofix',
        action='store_true',
        dest='autofix',
        help='Automatically fixes encountered not-pretty-formatted files',
    )
    parser.add_argument(
        '--aosp',
        action='store_true',
        dest='aosp',
        help='Formats Java code into AOSP format',
    )

    parser.add_argument('filenames', nargs='*', help='Filenames to fix')
    args = parser.parse_args(argv)

    google_java_formatter_jar = download_google_java_formatter_jar()

    status, output = run_command(
        'java -jar {} --set-exit-if-changed{} {} {}'.format(
            google_java_formatter_jar,
            ' --aosp' if args.aosp else '',
            '--replace' if args.autofix else '--dry-run',
            ' '.join(set(args.filenames)),
        ),
    )

    if output:
        print(
            '{}: {}'.format(
                'The following files have been fixed by google-java-formatter' if args.autofix else 'The following files are not properly formatted',  # noqa
                ', '.join(output.splitlines()),
            ),
        )

    return 0 if status == 0 else 1


if __name__ == '__main__':
    sys.exit(pretty_format_java())
